import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import VisuallyHidden from "@reach/visually-hidden";
import { Text } from "./Text";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
const sizes = () => ({
    xs: css({ width: "1.25rem", height: "1.25rem", fontSize: "0.7rem" }),
    sm: css({ width: "2.02rem", height: "2.02rem", fontSize: "0.875rem" }),
    md: css({ width: "3.27rem", height: "3.27rem", fontSize: "1.41rem" }),
    lg: css({ width: "5.29rem", height: "5.29rem", fontSize: "2.29rem" }),
    xl: css({ width: "8.57rem", height: "8.57rem", fontSize: "3.70rem" })
});
/**
 * Display a profile image to represent a user. Initials can be shown as a fallback
 */
export const Avatar = (_a) => {
    var { src, name, size = "md", srcSet } = _a, other = __rest(_a, ["src", "name", "size", "srcSet"]);
    const theme = useTheme();
    const dark = theme.colors.mode === "dark";
    const colors = Object.keys(theme.colors.palette);
    const img = src || srcSet;
    let initials = getInitials(name);
    if (size === "xs")
        initials = initials.substring(0, 1);
    const color = colors[Math.abs(makeHash(name)) % colors.length];
    const [error, setError] = React.useState(false);
    function onError() {
        setError(true);
    }
    return (_jsx("div", Object.assign({ css: [
            {
                display: "flex",
                alignItems: "center",
                flex: "0 0 auto",
                justifyContent: "center",
                borderRadius: "50%",
                backgroundColor: img
                    ? theme.colors.background.tint2
                    : theme.colors.palette[color][dark ? "light" : "base"]
            },
            sizes()[size]
        ] }, other, { children: img && !error ? (_jsx("img", { alt: name, src: src, srcSet: srcSet, onError: onError, css: {
                width: "100%",
                height: "100%",
                objectFit: "cover",
                borderRadius: "50%",
                display: "block"
            } }, void 0)) : (_jsxs("div", { children: [_jsx(VisuallyHidden, { children: name }, void 0),
                _jsx(Text, Object.assign({ css: {
                        fontWeight: size === "xs" ? 500 : 400,
                        fontSize: "inherit",
                        color: dark ? "rgba(0,0,0,0.75)" : "white"
                    } }, { children: initials }), void 0)] }, void 0)) }), void 0));
};
Avatar.propTypes = {
    src: PropTypes.string,
    name: PropTypes.string,
    size: PropTypes.oneOf(["xs", "sm", "md", "lg", "xl"]),
    srcSet: PropTypes.string
};
function getInitials(name = "?") {
    return name
        .replace(/\s+/, " ")
        .split(" ")
        .slice(0, 2)
        .map(v => v[0])
        .join("")
        .toUpperCase();
}
function makeHash(name = "?") {
    var hash = 0, i, chr;
    if (name.length === 0)
        return hash;
    for (i = 0; i < name.length; i++) {
        chr = name.charCodeAt(i);
        hash = (hash << 5) - hash + chr;
        hash |= 0; // Convert to 32bit integer
    }
    return hash;
}
//# sourceMappingURL=Avatar.js.map