import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { useUid } from "./Hooks/use-uid";
import { safeBind } from "./Hooks/compose-bind";
import { useTheme } from "./Theme/Providers";
import { Text } from "./Text";
import { Touchable } from "./Touchable";
import { useMeasure } from "./Hooks/use-measure";
import { Layer } from "./Layer";
import Highlighter from "react-highlight-words";
import { InputBase } from "./Form";
import { usePopper } from 'react-popper';
export const ComboBoxContext = React.createContext(null);
export const ComboBox = ({ children, onSelect, autocomplete = false, query, onQueryChange, topDivStyle }) => {
    const inputRef = React.useRef(null);
    const listRef = React.useRef(null);
    const listId = `list${useUid()}`;
    const options = React.useRef([]);
    //替换掉const position = usePositioner() of ./Hooks/use-positioner"直接使用'react-popper'包。
    const [referenceElement, setReferenceElement] = React.useState(null);
    const [popperElement, setPopperElement] = React.useState(null);
    const [arrowElement, setArrowElement] = React.useState(null);
    const { styles, attributes } = usePopper(referenceElement, popperElement, {
        modifiers: [{ name: 'arrow', options: { element: arrowElement } }],
    });
    const [expanded, setExpanded] = React.useState(false);
    const [selected, setSelected] = React.useState(null);
    const inputSize = useMeasure(inputRef);
    /**
     * Handle input change
     */
    const onInputChange = React.useCallback((e) => {
        // potentially show popover
        setSelected(null);
        if (!expanded) {
            setExpanded(true);
        }
        onQueryChange(e.target.value);
    }, [expanded]);
    /**
     * Escape key pressed
     */
    const onEscape = React.useCallback(() => {
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Enter pressed or item clicked
     */
    const onItemSelect = React.useCallback(() => {
        // call the parent with the selected value?
        setExpanded(false);
        onSelect && onSelect(selected);
        setSelected(null);
    }, [selected]);
    /**
     * Get the currently active option index
     * */
    const getSelectedIndex = React.useCallback(() => {
        if (!selected)
            return -1;
        return options.current.indexOf(selected || "");
    }, [options, selected]);
    /**
     * Arrow up pressed
     */
    const onArrowUp = React.useCallback(() => {
        const opts = options.current;
        const i = getSelectedIndex();
        // on input? cycle to bottom
        if (i === -1) {
            setSelected(opts[opts.length - 1]);
            // select prev
        }
        else {
            setSelected(opts[i - 1]);
        }
    }, [getSelectedIndex]);
    /**
     * Arrow down pressed
     */
    const onArrowDown = React.useCallback(() => {
        const opts = options.current;
        const i = getSelectedIndex();
        // if last, cycle to first
        if (i + 1 === opts.length) {
            setSelected(opts[0]);
            // or next
        }
        else {
            setSelected(opts[i + 1]);
        }
    }, [getSelectedIndex, selected]);
    /**
     * Handle keydown events
     */
    const onKeyDown = React.useCallback((e) => {
        switch (e.key) {
            case "ArrowUp":
                e.preventDefault();
                onArrowUp();
                break;
            case "ArrowDown":
                e.preventDefault();
                onArrowDown();
                break;
            case "Escape":
                e.preventDefault();
                onEscape();
                break;
            case "Enter":
                e.preventDefault();
                onItemSelect();
                break;
        }
    }, [onArrowDown, onArrowUp, onEscape, onItemSelect]);
    /**
     * Handle blur events
     */
    const handleBlur = React.useCallback(() => {
        requestAnimationFrame(() => {
            const focusedElement = document.activeElement;
            const list = listRef.current;
            if (focusedElement == inputRef.current || focusedElement == list) {
                // ignore
                return;
            }
            // ignore if our popover contains the focused element
            if (list && list.contains(focusedElement)) {
                return;
            }
            // hide popover
            setExpanded(false);
            setSelected(null);
        });
    }, []);
    /**
     * handle input focus
     */
    const handleFocus = React.useCallback(() => {
        setExpanded(true);
    }, []);
    /**
     * Handle item clicks
     */
    const handleOptionSelect = React.useCallback((value) => {
        onSelect && onSelect(value);
        setExpanded(false);
        setSelected(null);
    }, []);
    /**
     * Make a unique hash for list + option
     */
    const makeHash = React.useCallback((i) => {
        return listId + i;
    }, [listId]);
    return (_jsx(ComboBoxContext.Provider, Object.assign({ value: {
            listId,
            inputRef,
            listRef,
            options,
            onInputChange,
            selected,
            onKeyDown,
            handleBlur,
            handleFocus,
            handleOptionSelect,
            position: { setReferenceElement, setPopperElement, setArrowElement, styles, attributes },
            makeHash,
            expanded,
            inputSize: inputSize.bounds,
            query,
            autocomplete,
            topDivStyle,
        } }, { children: children }), void 0));
};
//分解配套的模式ComboBoxInput必需加到ComboBox底下的，ComboBox本身没有实际的div元素,靠context管理传递参数。
export const ComboBoxInput = (_a) => {
    var { component: Component = InputBase } = _a, other = __rest(_a, ["component"]);
    const context = React.useContext(ComboBoxContext);
    const [localValue, setLocalValue] = React.useState("");
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    //这里topDivStyle样式从上级组件ComboBox以共享同一个context模式传递的，算是第二次传递。
    const { onKeyDown, makeHash, selected, position, handleBlur, query, autocomplete, handleFocus, onInputChange, listId, inputRef, topDivStyle } = context;
    /** support autocomplete on selection */
    React.useEffect(() => {
        if (!autocomplete) {
            return;
        }
        if (selected) {
            setLocalValue(selected);
        }
        else {
            setLocalValue("");
        }
    }, [selected, autocomplete]);
    return (_jsx(Component, Object.assign({ id: listId, onKeyDown: onKeyDown, onChange: onInputChange, onBlur: handleBlur, onFocus: handleFocus, "aria-controls": listId, autoComplete: "off", value: localValue || query, "aria-readonly": true, "aria-autocomplete": "list", role: "textbox", css: [
            {},
            topDivStyle
        ], "aria-activedescendant": selected ? makeHash(selected) : undefined }, safeBind({
        ref: inputRef
    }, {
        ref: position.setReferenceElement
    }, other)), void 0));
};
export const ComboBoxList = (_a) => {
    var { children, autoHide = true } = _a, other = __rest(_a, ["children", "autoHide"]);
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { inputSize, expanded, listId, handleBlur, listRef, position, options } = context;
    React.useLayoutEffect(() => {
        options.current = [];
        return () => {
            options.current = [];
        };
    });
    return (_jsx(React.Fragment, { children: (expanded || !autoHide) && (_jsxs(Layer, Object.assign({ tabIndex: -1, elevation: "sm", style: position.styles.popper, "data-placement": position.attributes.popper, id: listId, role: "listbox", onBlur: handleBlur, className: "ComboBoxList", css: {
                overflow: "hidden",
                borderRadius: theme.radii.sm,
                outline: "none",
                width: inputSize.width +
                    inputSize.left +
                    (inputSize.right - inputSize.width) +
                    "px",
                margin: 0,
                padding: 0,
                zIndex: theme.zIndices.overlay,
            } }, safeBind({
            ref: listRef
        }, {
            ref: position.setPopperElement
        }, other), { children: [children, _jsx("div", { ref: position.setArrowElement, style: position.styles.arrow }, void 0)] }), "1")) }, void 0));
};
export const ComboBoxOption = (_a) => {
    var { value, children } = _a, other = __rest(_a, ["value", "children"]);
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { makeHash, handleOptionSelect, options, selected } = context;
    React.useEffect(() => {
        if (options.current) {
            options.current.push(value);
        }
    });
    const isSelected = selected === value;
    const onClick = React.useCallback(() => {
        handleOptionSelect(value);
    }, [value]);
    return (_jsx(Touchable, Object.assign({ tabIndex: -1, id: makeHash(value), role: "option", component: "li", onPress: onClick, "aria-selected": isSelected ? "true" : "false", css: {
            outline: "none",
            width: "100%",
            boxSizing: "border-box",
            display: "block",
            listStyleType: "none",
            margin: 0,
            padding: `0.5rem 0.75rem`,
            cursor: "pointer",
            background: isSelected ? theme.colors.background.tint1 : "none",
            "&.Touchable--hover": {
                background: theme.colors.background.tint1
            },
            "&.Touchable--active": {
                background: theme.colors.background.tint2
            }
        } }, other, { children: children || _jsx(ComboBoxOptionText, { value: value }, void 0) }), void 0));
};
export const ComboBoxOptionText = (_a) => {
    var { value } = _a, other = __rest(_a, ["value"]);
    const context = React.useContext(ComboBoxContext);
    const theme = useTheme();
    if (!context) {
        throw new Error("ComboBoxInput must be wrapped in a ComboBox component");
    }
    const { query } = context;
    return (_jsx(Text, Object.assign({ className: "ComboBoxOptionText", css: { fontWeight: theme.fontWeights.heading } }, other, { children: _jsx(Highlighter, { highlightStyle: {
                color: theme.colors.text.selected,
                background: "transparent"
            }, className: "ComboBoxOptionText__Highlighter", searchWords: [query], autoEscape: true, textToHighlight: value }, void 0) }), void 0));
};
//# sourceMappingURL=ComboBox.js.map