/** @jsxImportSource @emotion/react */
import { SerializedStyles } from "@emotion/react";
import * as React from "react";
export declare type InputSize = "sm" | "md" | "lg";
export interface InputGroupProps extends React.HTMLAttributes<HTMLDivElement> {
    id?: string;
    /** A label is required for accessibility purposes. Use `hideLabel` to hide it. */
    label: string;
    /** Visually hide the label. It remains accessible to screen readers. */
    hideLabel?: boolean;
    error?: string | React.ReactNode;
    /** Optional help text */
    helpText?: string;
    /** A single input element */
    children?: React.ReactNode;
    labelStyle?: SerializedStyles;
    labelTextStyle?: SerializedStyles;
}
export declare const InputGroup: React.FunctionComponent<InputGroupProps>;
export interface InputBaseProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** The size of the input element */
    inputSize?: InputSize;
    topDivStyle?: SerializedStyles;
}
/**
 * Our basic Input element. Use this when building customized
 * forms. Otherwise, stick with InputGroup
 */
export declare const InputBase: React.ForwardRefExoticComponent<InputBaseProps & React.RefAttributes<HTMLInputElement>>;
export interface InputProps extends InputBaseProps {
    /** 控制是否满上宽度;
     *  需要在<input> 上 去控制大尺寸上限的width:  ，以及自适应屏幕大小后的 max-width: 缩小尺寸。
     * */
    fullWidth?: boolean;
}
export declare const Input: React.ForwardRefExoticComponent<InputProps & React.RefAttributes<HTMLInputElement>>;
export interface TextAreaProps extends React.TextareaHTMLAttributes<HTMLTextAreaElement> {
    /** The size of the textarea element */
    inputSize?: InputSize;
    topDivStyle?: SerializedStyles;
}
/**
 * Textarea version of InputBase
 */
export declare const TextArea: React.FunctionComponent<TextAreaProps>;
export interface LabelProps extends React.LabelHTMLAttributes<HTMLLabelElement> {
    hide?: boolean;
    htmlFor: string;
    textStyle?: SerializedStyles;
}
/**
 * A styled Label to go along with input elements
 */
export declare const Label: React.FunctionComponent<LabelProps>;
export interface SelectProps extends React.SelectHTMLAttributes<HTMLSelectElement> {
    /** The size of the select box */
    inputSize?: InputSize;
    divStyle?: SerializedStyles;
    topDivStyle?: SerializedStyles;
}
/**
 * A styled select
 * 若触摸屏 不能支持multiple形态的的Select，只能单选。
 * multiple 只是摊开选择列表，onChange  .target.value无法提供多选数组，必须额外维护可以多选的被选中状态和数据数组。
 */
export declare const Select: React.FunctionComponent<SelectProps>;
export interface CheckProps extends React.InputHTMLAttributes<HTMLInputElement> {
    /** A label for the checkmark. */
    label: string;
    topDivStyle?: SerializedStyles;
}
export declare const Check: React.FunctionComponent<CheckProps>;
export interface SuffixInputProps extends InputBaseProps {
    textStyle?: SerializedStyles;
}
/**
 * A styled Label to go along with input elements
 * 若children有，就是有附带单位后缀串的模式；
 * 带单位后缀的说明，70%给输入框，后面30%给叙述单位字串{空格也算}
 * 输入的 单位说明 字符串 放在 <Text className="Suffix__text"  >
 */
export declare const SuffixInput: React.FunctionComponent<SuffixInputProps>;
export interface InputGroupLineProps extends InputGroupProps {
    lineStyle?: SerializedStyles;
    switchPx?: number;
}
export declare const InputGroupLine: React.FunctionComponent<InputGroupLineProps>;
