import * as React from "react";
/**
 * A Table provides a useful abstraction for managing rows and columns.
 */
interface TableProps extends React.HTMLAttributes<HTMLTableElement> {
    /** An optional minimum width for table content. */
    minWidth?: string;
    /** An optional array of fixed layout widths for each column */
    fixed?: string[];
}
export declare const Table: React.FunctionComponent<TableProps>;
/**
 * A TableHead is used to render column labels in a table.
 */
declare type TableHeadProps = React.HTMLAttributes<HTMLTableSectionElement>;
export declare function TableHead({ children, ...other }: TableHeadProps): import("@emotion/react/types/jsx-namespace").EmotionJSX.Element;
interface TableRowProps extends React.HTMLAttributes<HTMLTableRowElement> {
    /** A callback when a row is selected */
    onClick?: () => void;
}
export declare const TableRow: React.FunctionComponent<TableRowProps>;
declare type AlignCellType = "right" | "left" | "center" | "justify";
declare type tableCellVariants = "head" | "body";
declare type TableCellBaseProps = React.ThHTMLAttributes<HTMLTableHeaderCellElement> & React.TdHTMLAttributes<HTMLTableDataCellElement>;
interface TableCellProps extends TableCellBaseProps {
    align?: AlignCellType;
    variant?: tableCellVariants;
    ellipsis?: boolean;
    component?: React.ElementType<TableCellBaseProps>;
}
export declare const TableCell: React.FunctionComponent<TableCellProps>;
/**
 * TableBody - indicates the body (and scrollable) portion of our table.
 */
declare type TableBodyProps = React.HTMLAttributes<HTMLTableSectionElement>;
export declare function TableBody({ children, ...other }: TableBodyProps): import("@emotion/react/types/jsx-namespace").EmotionJSX.Element;
/**
 * An ExpandingRow displays additional content about the row when clicked.
 */
interface ExpandingRowProps {
    /** The expanded content to show when the user selects the row */
    content: (close: () => void) => React.ReactNode | React.ReactNode;
    children: React.ReactNode;
}
export declare const ExpandingRow: React.FunctionComponent<ExpandingRowProps>;
export {};
