import { __rest } from "tslib";
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Positioner } from "./Positions";
import { Text } from "./Text";
import { animated } from "react-spring";
import PropTypes from "prop-types";
import { isMobile } from "is-mobile";
import { useUid } from "./Hooks/use-uid";
import { useTheme } from "./Theme/Providers";
export const arrowStyles = (color) => css(`
  position: absolute;
  width: 3em;
  height: 3em;
  &[data-placement*='bottom'] {
    bottom: 100%;
    left: 0;
    margin-top: 0em;
    width: 1em;
    height: 0.25em;
    &::before {
      width: 10px;
      border: none;
      height: 10px;
      background: ${color};
      border-top: 1px solid ${color === "white" ? " #dee2e685" : color};
      border-right: 1px solid ${color === "white" ? "#dee2e685" : color};
      transform: rotate(-45deg);
      border-radius: 2px;
      margin-top: -1px;
    }
  }
  &[data-placement*='top'] {
    top: 100%;
    left: 0;
    margin-bottom: 0;
    width: 1em;
    height: 0.25em;
    &::before {
      border-width: 0.25em 0.25em 0 0.25em;
      border-color: ${color} transparent transparent transparent;
    }
  }
  &[data-placement*='right'] {
    right: 100%;
    height: 1em;
    width: 0.25em;
    &::before {
      border-width: 0.25em 0.25em 0.25em 0;
      border-color: transparent ${color} transparent transparent;
    }
  }
  &[data-placement*='left'] {
    left: 100%;
    height: 1em;
    width: 0.25em;
    &::before {
      border-width: 0.25em 0 0.25em 0.25em;
      border-color: transparent transparent transparent ${color};
    }
  }
  &::before {
    content: '';
    margin: auto;
    display: block;
    width: 0;
    height: 0;
    border-style: solid;
  }
`);
export const Tooltip = (_a) => {
    var { placement, children, content, delayIn = 0, delayOut = 0, hover: enableHover = true, maxWidth = "300px" } = _a, other = __rest(_a, ["placement", "children", "content", "delayIn", "delayOut", "hover", "maxWidth"]);
    const id = useUid();
    const theme = useTheme();
    const dark = theme.colors.mode === "dark";
    const [show, setShow] = React.useState(false);
    const [hovering, setHovering] = React.useState(false);
    const [delayShow, setDelayShow] = React.useState(false);
    function renderTrigger({ ref }) {
        // We don't want tooltips to show on touch based devices
        // so we just return the child without the event handlers.
        // Really not sure if this is the best way to do this.
        if (isMobile()) {
            return children;
        }
        const child = React.Children.only(children);
        // Enable on non-mobile devices
        return React.cloneElement(child, {
            ref,
            "aria-describedby": id,
            onMouseEnter: () => {
                if (!show)
                    setShow(true);
            },
            onMouseLeave: () => {
                if (show)
                    setShow(false);
            },
            onFocus: () => {
                if (!show)
                    setShow(true);
            },
            onBlur: () => {
                if (show)
                    setShow(false);
            }
        });
    }
    function onMouseLeave() {
        setHovering(false);
    }
    function onMouseEnter() {
        setHovering(true);
    }
    React.useEffect(() => {
        let timeoutIn;
        let timeoutOut;
        if (show) {
            timeoutIn = setTimeout(() => {
                setDelayShow(true);
            }, delayIn);
        }
        else {
            timeoutOut = setTimeout(() => {
                setDelayShow(false);
            }, delayOut);
        }
        return () => {
            if (timeoutIn) {
                clearInterval(timeoutIn);
            }
            if (timeoutOut) {
                clearInterval(timeoutOut);
            }
        };
    }, [delayIn, delayOut, show]);
    return (_jsx(Positioner, Object.assign({ placement: placement, isOpen: delayShow || (hovering && enableHover), target: renderTrigger }, { children: ({ placement, ref, style, arrowProps }, state) => (_jsxs(animated.div, Object.assign({ id: id, "data-placement": placement, role: "tooltip", className: "Tooltip", ref: ref, style: Object.assign(Object.assign({}, style), { opacity: state.opacity }), css: {
                zIndex: theme.zIndices.tooltip,
                margin: theme.spaces.xs
            }, onMouseEnter: onMouseEnter, onMouseLeave: onMouseLeave }, other, { children: [_jsx("div", { className: "Tooltip__content", "data-placement": placement, css: arrowStyles(dark
                        ? theme.colors.background.tint1
                        : theme.colors.palette.gray.dark), ref: arrowProps.ref, style: arrowProps.style }, void 0),
                _jsx(Text, Object.assign({ className: "Tooltip__text", variant: "body", css: [
                        {
                            fontSize: theme.fontSizes[0],
                            display: "inline-block",
                            margin: 0,
                            textAlign: "center",
                            maxWidth,
                            boxShadow: theme.shadows.md,
                            borderRadius: theme.radii.sm,
                            padding: `${theme.spaces.xs} ${theme.spaces.md}`,
                            color: "white",
                            background: dark
                                ? theme.colors.background.tint1
                                : theme.colors.palette.gray.dark
                        }
                    ] }, { children: content }), void 0)] }), void 0)) }), void 0));
};
Tooltip.propTypes = {
    content: PropTypes.node.isRequired,
    children: PropTypes.node.isRequired,
    hover: PropTypes.bool,
    delayIn: PropTypes.number,
    delayOut: PropTypes.number,
    placement: PropTypes.oneOf([
        "auto-start",
        "auto",
        "auto-end",
        "top-start",
        "top",
        "top-end",
        "right-start",
        "right",
        "right-end",
        "bottom-end",
        "bottom",
        "bottom-start",
        "left-end",
        "left",
        "left-start"
    ])
};
//# sourceMappingURL=Tooltip.js.map